#include <stdlib.h>
#include <stdint.h>

#include <png.h>


void save_png16L(const char *output_file_name, const int ni, const int nj,
                 const float bounds[6], uint16_t const*const*const pixels)
{

    // Open up a file for writing
	FILE* output = fopen(output_file_name, "wb");
	
	// Create a png pointer without any special callbacks
	png_structp png_ptr = png_create_write_struct(
	    PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
	    
	// Create an info pointer
	png_infop info_ptr = png_create_info_struct(png_ptr);
		
    // Set physical vars
	png_set_IHDR(png_ptr, info_ptr, ni, nj, 16, PNG_COLOR_TYPE_GRAY,
                 PNG_INTERLACE_NONE, PNG_COMPRESSION_TYPE_BASE,
                 PNG_FILTER_TYPE_BASE);
    
    png_text text[2];
        
    char zmin[10];
    snprintf(zmin, 10, "%g", bounds[2]);
    text[0].compression = PNG_TEXT_COMPRESSION_NONE;
    text[0].key = "zmin";
    text[0].text = zmin;

    char zmax[10];
    snprintf(zmax, 10, "%g", bounds[5]);
    text[1].compression = PNG_TEXT_COMPRESSION_NONE;
    text[1].key = "zmax";
    text[1].text = zmax;
    
    png_set_text(png_ptr, info_ptr, text, 2);
    
    // bounds should be mm in the order {xmin, ymin, zmin, xmax, ymax, zmax}
    png_set_pHYs(png_ptr, info_ptr,
                 1000 * ni / (bounds[3]-bounds[0]),
                 1000 * nj / (bounds[4]-bounds[1]),
                 PNG_RESOLUTION_METER);
    
    // Write the PNG to file
	png_init_io(png_ptr, output);
	png_set_rows(png_ptr, info_ptr, (png_bytepp)pixels);
	png_write_png(png_ptr, info_ptr, PNG_TRANSFORM_SWAP_ENDIAN, NULL);
    fclose(output);
   
    png_destroy_write_struct(&png_ptr, &info_ptr);
}

void downsample_image(int ni, int nj,
                      uint16_t const*const*const in,
                      uint8_t* const*const out)
{
    for (int j=0; j < nj; ++j)
        for (int i=0; i < ni; ++i)
            out[j][i] = in[j][i] >> 8;
}

void triple_image(int ni, int nj,
                  uint8_t const*const*const in,
                  uint8_t (**out)[3])
{
    for (int j=0; j < nj; ++j)
        for (int i=0; i < ni; ++i) {
            out[j][i][0]   = in[j][i];
            out[j][i][1] = in[j][i];
            out[j][i][2] = in[j][i];
        }
}


void count_by_color(char* image, int w, int h, uint32_t maxindex, uint32_t* count)
{
    uint32_t a=0;
    for (int j=0; j < h; ++j) {
        for (int i=0; i < w; ++i) {
            uint8_t r = image[a++];
            uint8_t g = image[a++];
            uint8_t b = image[a++];
            
            uint32_t d = (r << 16) | (g << 8) | b;
            
            if (d > 0) {
                if (d - 1 >= maxindex) {
                    printf("UHOH! %i %i %i %u\n", r,g,b,d - 1);
                } else {
                    count[d-1]++;
                }
            }
        }
    }
}