import ctypes

from vec3f import Vec3f

class Region(ctypes.Structure):
    _fields_ = [('imin', ctypes.c_uint32),
                ('jmin', ctypes.c_uint32),
                ('kmin', ctypes.c_uint32),
                ('ni', ctypes.c_uint32),
                ('nj', ctypes.c_uint32),
                ('nk', ctypes.c_uint32),
                ('voxels', ctypes.c_uint64),
                ('X', ctypes.POINTER(ctypes.c_float)),
                ('Y', ctypes.POINTER(ctypes.c_float)),
                ('Z', ctypes.POINTER(ctypes.c_float)),
                ('L', ctypes.POINTER(ctypes.c_uint16))]

    def __init__(self, (xmin, ymin, zmin)=(0.,0.,0.),
                       (xmax, ymax, zmax)=(0.,0.,0.),
                       scale=100., dummy=False):
                   
        dx = float(xmax - xmin)
        dy = float(ymax - ymin)
        dz = float(zmax - zmin)
        
        ni = max(int(dx*scale), 1)
        nj = max(int(dy*scale), 1)
        nk = max(int(dz*scale), 1)
        
        ctypes.Structure.__init__(self,
                                  0, 0, 0,
                                  ni, nj, nk,
                                  ni*nj*nk,
                                  None, None, None, None)
        
        if dummy is False:
            libtree.build_arrays(ctypes.byref(self),
                                 xmin, ymin, zmin,
                                 dx, dy, dz)
            self.head = True
            
    def __del__(self):
        if hasattr(self, 'head'):
            libtree.free_arrays(self)

    def __repr__(self):
        return ('[(%g, %g), (%g, %g), (%g, %g)]' %
            (self.imin, self.imin + self.ni,
             self.jmin, self.jmin + self.nj,
             self.kmin, self.kmin + self.nk))
            
    def split(self, count=2):            
        L = (Region*count)()
        count = libtree.split(self, L, count)
        return L[:count]

    def split_xy(self, count=2):       
        L = (Region*count)()
        count = libtree.split_xy(self, L, count)
        return L[:count]

    def octsect(self, all=False, overlap=False):
        L = (Region*8)()
        if overlap:
            bits = libtree.octsect_overlap(self, L)
        else:
            bits = libtree.octsect(self, L)
            
        if all:
            return [L[i] if (bits & (1 << i)) else None for i in range(8)]
        else:
            return [L[i] for i in range(8) if (bits & (1 << i))]
            
    def deproject(self, M):
        a = (ctypes.POINTER(Vec3f)*self.nj)()
        for j in range(self.nj):
            a[j] = (Vec3f*self.ni)()
        libtree.deproject_region(self, M, a)
        return a

from libtree import libtree