import math

from koko.c.tree import *
import koko.clib.shapes2d as s2d

################################################################################

def extrusion(part, z0, z1):
    s = MathTree.max(part, MathTree.max(z0 - Z, Z - z1))
    s.shape = True
    return s

def cylinder(x0, y0, z0, z1, r):
    return extrusion(s2d.circle(x0, y0, r), z0, z1)

def sphere(x0, y0, z0, r):
    s = MathTree.sqrt(MathTree.pow(X,2) +
                      MathTree.pow(Y,2) +
                      MathTree.pow(Z,2)) - r
    s.shape = True
    return move(s, x0, y0, z0)

def cube(x0, x1, y0, y1, z0, z1):
    s = MathTree.max(MathTree.max(MathTree.max(x0 - X, X - x1),
                                  MathTree.max(y0 - Y, Y - y1)),
                     MathTree.max(z0 - Z, Z - z1))
    s.shape = True
    return s

def cone(x0, y0, z0, z1, r):
    x0 = MathTree.Constant(x0)
    y0 = MathTree.Constant(y0)
    z0 = MathTree.Constant(z0)
    z1 = MathTree.Constant(z1)
    cyl = cylinder(x0, y0, z0, z1, r)
    return taper_xy_z(cyl, x0, y0, z0, z1, 1.0, 0.0)

def pyramid(x0, x1, y0, y1, z0, z1):
    x0 = MathTree.Constant(x0)
    x1 = MathTree.Constant(x1)
    y0 = MathTree.Constant(y0)
    y1 = MathTree.Constant(y1)
    z0 = MathTree.Constant(z0)
    z1 = MathTree.Constant(z1)
    c = cube(x0, x1, y0, y1, z0, z1)
    return taper_xy_z(c, (x0+x1)/2., (y0+y1)/2., z0, z1, 1.0, 0.0)

################################################################################

move = s2d.move

def rotate_x(part, angle):
    angle *= math.pi/180
    
    return part.map(Y= math.cos(angle)*Y+math.sin(angle)*Z,
                    Z=-math.sin(angle)*Y+math.cos(angle)*Z)
                    
def rotate_y(part, angle):
    angle *= math.pi/180
    
    return part.map(X= math.cos(angle)*X+math.sin(angle)*Z,
                    Z=-math.sin(angle)*X+math.cos(angle)*Z)
                    
rotate_z = s2d.rotate

################################################################################

reflect_x = s2d.reflect_x
reflect_y = s2d.reflect_y

def reflect_z(part, z0=0):
    if z0:
        return part.map(Z=2*z0-Z)
    else:
        return part.map(Z=-Z)
        
reflect_xy = s2d.reflect_xy
def reflect_xz(part):
    return part.map(X=Z, Z=X)

def reflect_yz(part):
    return part.map(Y=Z, Z=Y)
    
################################################################################

scale_x = s2d.scale_x
scale_y = s2d.scale_y

def scale_z(part, z0, sz):
    if sz == 1:
        return part
    if z0 == 0:
        return part.map(Z=Z/sz)
    return part.map(Z=z0 + (Z-z0)/sz)
    
################################################################################

shear_x_y = s2d.shear_x_y

def shear_x_z(part, z0, z1, dx0, dx1):
    return part.map(X=X-dx0-(dx1-dx0)*(Z-z0)/(z1-z0))
    
################################################################################

taper_x_y = s2d.taper_x_y

def taper_xy_z(part, x0, y0, z0, z1, s0, s1):
    t = (z1 - z0) / (s1*(Z - z0) + s0*(z1 - Z))
    newX = x0 + (X - x0) * t
    newY = y0 + (Y - y0) * t
    return part.map(X=newX, Y=newY)
    
################################################################################

def revolve_y(part):
    ''' Revolve a part in the XY plane about the Y axis. '''
    return part.map(X=MathTree.sqrt(MathTree.pow(X,2)+MathTree.pow(Z,2)))
    
def revolve_x(part):
    ''' Revolve a part in the XY plane about the X axis. '''
    return part.map(Y=MathTree.sqrt(MathTree.pow(Y,2)+MathTree.pow(Z,2)))